@@ -133,11 +133,11 @@ void BonusDisciplineWindow::ChooseSlot()
                     foreach (QString w, m_Elements)
                         wm->Weapons.append(Weapon::GetClass(w));
                 }else if (name == "Psi-surge"){
-                    d->Active = false;
+                    d->SetIsActive(false);
                     // Add also Mindblast:
                     if (!m_Player->HasDiscipline("Mindblast")){
                         Mindblast* m = new Mindblast(m_Player);
-                        m->Active = false;
+                        m->SetIsActive(false);
                         m->Serie = 0; // To avoid counting it in GetDisciplineCount
                         m_Player->KaiDisciplines.append(m);
                     }
@@ -238,11 +238,11 @@ void BonusDisciplineWindow::OkSlot()
                     foreach (QString w, m_Elements)
                         wm->Weapons.append(Weapon::GetClass(w));
                 }else if (d->Name == "Psi-surge"){
-                    d->Active = false;
+                    d->SetIsActive(false);
                     // Add also Mindblast:
                     if (!m_Player->HasDiscipline("Mindblast")){
                         Mindblast* m = new Mindblast(m_Player);
-                        m->Active = false;
+                        m->SetIsActive(false);
                         m->Serie = 0; // To avoid counting it in GetDisciplineCount
                         m_Player->KaiDisciplines.append(m);
                     }
