@@ -3,7 +3,7 @@
 #include <QLabel>
 #include <QPushButton>
 
-ChooseItemWindow::ChooseItemWindow(QWidget *parent, Player* player, int mode) :
+ChooseItemWindow::ChooseItemWindow(QWidget *parent, Player* player, int mode, bool mandatory) :
     DialogWindow(parent),
     ui(new Ui::ChooseItemWindow)
 {
@@ -14,10 +14,30 @@ ChooseItemWindow::ChooseItemWindow(QWidget *parent, Player* player, int mode) :
     ui->lblMessage->setVisible(false);
 #endif
 
-    SetCanClose(false);
+    SetCanClose(!mandatory);
     m_Selected = NULL;
     m_Player = player;
     m_Mode = mode;
+
+    switch(mode){
+    case ChooseWeapon:
+    case ChooseWeaponSpecial:
+        setWindowTitle("Select a weapon");
+        break;
+    case ChooseItem:
+    case ChooseItemExceptMeals:
+        setWindowTitle("Select a item");
+        break;
+    case ChooseMeal:
+        setWindowTitle("Select a meal");
+        break;
+    case ChooseSpecialItem:
+        setWindowTitle("Select a special item");
+        break;
+    }
+
+    ui->pbnCancel->setVisible(!mandatory);
+    connect(ui->pbnCancel, SIGNAL(clicked()), this, SLOT(CancelSlot()));
     ShowItems();
 }
 
@@ -49,7 +69,10 @@ void ChooseItemWindow::ShowItems()
         ShowSpecialItems();
         break;
     case ChooseWeapon:
-        ShowWeapons();
+        ShowWeapons(false);
+        break;
+    case ChooseWeaponSpecial:
+        ShowWeapons(true);
         break;
     default:
         break;
@@ -58,7 +81,7 @@ void ChooseItemWindow::ShowItems()
 
 void ChooseItemWindow::ShowMeals()
 {
-    setWindowTitle("Choose a Meal to eat");
+    setWindowTitle("Choose a meal to eat");
     int row=0;
     int index = 0;
     QSizePolicy stretchZero;
@@ -96,7 +119,7 @@ void ChooseItemWindow::ShowMeals()
 
 void ChooseItemWindow::ShowBackpackItems(bool excludeMeals)
 {
-    setWindowTitle("Choose an Item");
+    setWindowTitle("Choose a item");
     int row=0;
     int index = 0;
     QSizePolicy stretchZero;
@@ -118,7 +141,7 @@ void ChooseItemWindow::ShowBackpackItems(bool excludeMeals)
             ui->layItems->addWidget(lbl, row, 1);
 
             lbl = new QLabel(this);
-            lbl->setText(i->Description);
+            lbl->setText(i->GetDescription());
             lbl->setWordWrap(true);
             ui->layItems->addWidget(lbl, row, 2);
 
@@ -137,7 +160,7 @@ void ChooseItemWindow::ShowBackpackItems(bool excludeMeals)
 
 void ChooseItemWindow::ShowSpecialItems()
 {
-    setWindowTitle("Choose an Item");
+    setWindowTitle("Choose a item");
     int row=0;
     int index = 0;
     QSizePolicy stretchZero;
@@ -156,7 +179,7 @@ void ChooseItemWindow::ShowSpecialItems()
             ui->layItems->addWidget(lbl, row, 1);
 
             lbl = new QLabel(this);
-            lbl->setText(i->Description);
+            lbl->setText(i->GetDescription());
             lbl->setWordWrap(true);
             ui->layItems->addWidget(lbl, row, 2);
 
@@ -173,15 +196,15 @@ void ChooseItemWindow::ShowSpecialItems()
     }
 }
 
-void ChooseItemWindow::ShowWeapons()
+void ChooseItemWindow::ShowWeapons(bool special)
 {
-    setWindowTitle("Choose a Weapon");
+    setWindowTitle("Choose a weapon");
     int row=0;
     int index = 0;
     QSizePolicy stretchZero;
     stretchZero.setHorizontalStretch(0);
 
-    foreach(Weapon* w, m_Player->Weapons()){
+    foreach(Weapon* w, m_Player->Weapons(special)){
         QLabel* lbl = new QLabel(this);
         lbl->setSizePolicy(stretchZero);
         lbl->setText(QString("<b>%1:</b>").arg(row + 1));
@@ -193,7 +216,7 @@ void ChooseItemWindow::ShowWeapons()
         ui->layItems->addWidget(lbl, row, 1);
 
         lbl = new QLabel(this);
-        lbl->setText(w->Description);
+        lbl->setText(w->GetDescription());
         lbl->setWordWrap(true);
         ui->layItems->addWidget(lbl, row, 2);
 
@@ -224,9 +247,17 @@ void ChooseItemWindow::SelectedSlot()
     case ChooseWeapon:
         m_Selected = m_Player->Weapons().at(idx);
         break;
+    case ChooseWeaponSpecial:
+        m_Selected = m_Player->Weapons(true).at(idx);
+        break;
     default:
         break;
     }
 
     accept();
 }
+
+void ChooseItemWindow::CancelSlot()
+{
+    reject();
+}
