@@ -85,7 +85,7 @@ void LoreCircles::ClearCircles()
 KaiDiscipline::KaiDiscipline(QObject *parent) :
     QObject(parent)
 {
-    Active = true;
+    m_Active = true;
     Name = "";
     HelpText = "";
     CombatSkillDelta = 0;
@@ -95,7 +95,7 @@ KaiDiscipline::KaiDiscipline(QObject *parent) :
 KaiDiscipline::KaiDiscipline(const KaiDiscipline& discipline) :
     QObject(discipline.parent())
 {
-    Active = discipline.Active;
+    m_Active = discipline.m_Active;
     Name = discipline.Name;
 	Serie = discipline.Serie;
     HelpText = discipline.HelpText;
