@@ -2,6 +2,20 @@
 #include <QDebug>
 #include <stdlib.h>
 
+QStringList Player::RanksKai = QStringList() << "Novice" << "Intuite" << "Doan" << "Acolyte"
+                                             << "Initiate" << "Aspirant" << "Guardian" << "Warmarn"
+                                             << "Savant" << "Master";
+
+QStringList Player::RanksMagnakai = QStringList() << "Kai Master" << "Kai Master Senior" << "Kai Master Superior"
+                                                  << "Primate" << "Tutelary" << "Principalin" << "Mentora"
+                                                  << "Scion-kai" << "Archmaster" << "Kai Grand Master";
+
+QStringList Player::RanksGrandMaster = QStringList() << "Kai Grand Master Senior" << "Kai Grand Master Superior"
+                                                     << "Kai Grand Sentinel" << "Kai Grand Defender"
+                                                     << "Kai Grand Guardian" << "Sun Knight" << "Sun Lord"
+                                                     << "Sun Thane" << "Grand Thane" << "Grand Crown" << "Sun Prince"
+                                                     << "Kai Supreme Master";
+
 Player::Player(QObject *parent) :
     QObject(parent)
 {
@@ -35,7 +49,10 @@ Player::Player(const Player& player) :
 QList<Weapon*> Player::Weapons(bool includeSpecial)
 {
     QList<Weapon*> res;
-    foreach (Item* i, Items){
+
+    QList<Item*>::const_iterator iterator;
+    for (iterator = Items.constBegin(); iterator != Items.constEnd(); ++iterator){
+        Item* i = *iterator;
         if (i->GetIsWeapon() && ((!includeSpecial && !i->GetIsSpecial()) || includeSpecial))
             res.append((Weapon*)i);
     }
@@ -45,7 +62,9 @@ QList<Weapon*> Player::Weapons(bool includeSpecial)
 QList<Item*> Player::BackpackItems()
 {
     QList<Item*> res;
-    foreach (Item* i, Items){
+    QList<Item*>::const_iterator iterator;
+    for (iterator = Items.constBegin(); iterator != Items.constEnd(); ++iterator){
+        Item* i = *iterator;
         if (i->GetIsBackpackItem())
             res.append(i);
     }
@@ -55,7 +74,9 @@ QList<Item*> Player::BackpackItems()
 QList<Item*> Player::SpecialItems()
 {
     QList<Item*> res;
-    foreach (Item* i, Items){
+    QList<Item*>::const_iterator iterator;
+    for (iterator = Items.constBegin(); iterator != Items.constEnd(); ++iterator){
+        Item* i = *iterator;
         if (i->GetIsSpecial())
             res.append(i);
     }
@@ -64,43 +85,57 @@ QList<Item*> Player::SpecialItems()
 
 QString Player::GetRankDescription(int serie, int kaiDisciplinesCount)
 {
-    QStringList kai = QStringList() << "Novice" << "Intuite" << "Doan" << "Acolyte"
-                                    << "Initiate" << "Aspirant" << "Guardian" << "Warmarn"
-                                    << "Savant" << "Master";
-
-    QStringList magnakai = QStringList() << "Kai Master" << "Kai Master Senior" << "Kai Master Superior"
-                                         << "Primate" << "Tutelary" << "Principalin" << "Mentora"
-                                         << "Scion-kai" << "Archmaster" << "Kai Grand Master";
-
-    QStringList grandMaster = QStringList() << "Kai Grand Master Senior" << "Kai Grand Master Superior"
-                                            << "Kai Grand Sentinel" << "Kai Grand Defender"
-                                            << "Kai Grand Guardian" << "Sun Knight" << "Sun Lord"
-                                            << "Sun Thane" << "Grand Thane" << "Grand Crown" << "Sun Prince"
-                                            << "Kai Supreme Master";
-
-
     switch (serie){
     case Book::Kai:
-        return kai.at(kaiDisciplinesCount - 1);
+        return RanksKai.at(kaiDisciplinesCount - 1);
         break;
     case Book::Magnakai:
-        return magnakai.at(kaiDisciplinesCount - 1);
+        return RanksMagnakai.at(kaiDisciplinesCount - 1);
         break;
     case Book::GrandMaster:
-        return grandMaster.at(kaiDisciplinesCount - 1);
+        return RanksGrandMaster.at(kaiDisciplinesCount - 1);
         break;
     case Book::NewOrder:
-        return grandMaster.at(kaiDisciplinesCount - 4);
+        return RanksGrandMaster.at(kaiDisciplinesCount - 4);
         break;
     }
 
     return "";
 }
 
+int Player::GetRankDisciplines(int serie, QString rank)
+{
+    int iRank = 0;
+    switch (serie){
+    case Book::Kai:
+        iRank = RanksKai.indexOf(rank);
+        break;
+    case Book::Magnakai:
+        iRank = RanksMagnakai.indexOf(rank);
+        break;
+    case Book::GrandMaster:
+        iRank = RanksGrandMaster.indexOf(rank);
+        break;
+    case Book::NewOrder:
+        iRank = RanksGrandMaster.indexOf(rank);
+        break;
+    }
+
+    return iRank + 1;
+}
+
+bool Player::HasRank(QString rank)
+{
+    int iRank = GetRankDisciplines(GetSerie(), rank);
+    return GetDisciplineCount() >= iRank;
+}
+
 int Player::MealsCount()
 {
     int result = 0;
-    foreach (Item* i, Items){
+    QList<Item*>::const_iterator iterator;
+    for (iterator = Items.constBegin(); iterator != Items.constEnd(); ++iterator){
+        Item* i = *iterator;
         if (i->GetIsMeal())
             result += 1;
     }
@@ -139,7 +174,9 @@ int Player::GetCombatSkill(Enemy* enemy, QStringList* description)
 
     int result = BaseCombatSkill + TemporaryCombatSkillDelta;
 
-    foreach (Item* i, Items){
+    QList<Item*>::const_iterator iterator;
+    for (iterator = Items.constBegin(); iterator != Items.constEnd(); ++iterator){
+        Item* i = *iterator;
         if (i->GetIsEquipped()){
             result += i->GetCombatskillDelta();
             if (description && i->GetCombatskillDelta())
@@ -244,7 +281,9 @@ int Player::GetBaseEndurance(QStringList* description)
         description->append(QString("Base|%1").arg(BaseEndurance));
 
     int result = BaseEndurance;
-    foreach (Item* i, Items){
+    QList<Item*>::const_iterator iterator;
+    for (iterator = Items.constBegin(); iterator != Items.constEnd(); ++iterator){
+        Item* i = *iterator;
         if (i->GetIsEquipped()){
             result += i->GetEnduranceDelta();
             if (description && i->GetEnduranceDelta())
@@ -449,7 +488,9 @@ void Player::SetGoldCrowns(int crowns)
 int Player::GetMaxArrows()
 {
     int res = 0;
-    foreach (Item* i, Items){
+    QList<Item*>::const_iterator iterator;
+    for (iterator = Items.constBegin(); iterator != Items.constEnd(); ++iterator){
+        Item* i = *iterator;
         if (i->GetIsSpecial() && i->GetIsQuiver())
             res += 6;
     }
@@ -459,7 +500,9 @@ int Player::GetMaxArrows()
 int Player::GetArrows()
 {
     int res = 0;
-    foreach (Item* i, Items){
+    QList<Item*>::const_iterator iterator;
+    for (iterator = Items.constBegin(); iterator != Items.constEnd(); ++iterator){
+        Item* i = *iterator;
         if (i->GetIsSpecial() && i->GetIsQuiver())
             res += i->GetArrows();
     }
@@ -470,7 +513,9 @@ void Player::SetArrows(int value)
 {
     int quivers = 0;
     int arrows = 0;
-    foreach (Item* i, Items){
+    QList<Item*>::const_iterator iterator;
+    for (iterator = Items.constBegin(); iterator != Items.constEnd(); ++iterator){
+        Item* i = *iterator;
         if (i->GetIsSpecial() && i->GetIsQuiver()){
             quivers++;
             arrows += i->GetArrows();
@@ -486,7 +531,8 @@ void Player::SetArrows(int value)
         value = 0;
 
     int delta = value - arrows;
-    foreach (Item* i, Items){
+    for (iterator = Items.constBegin(); iterator != Items.constEnd(); ++iterator){
+        Item* i = *iterator;
         if (i->GetIsSpecial() && i->GetIsQuiver()){
             int absDelta = abs(delta);
             if (delta < 0){
@@ -558,7 +604,6 @@ Item* Player::AddHealingItem(QString name, int enduranceRestore)
         Item* i = new Item(this);
         i->Name = name;
         i->SetEnduranceRestore(enduranceRestore);
-        i->Description = QString("Restores %1 ENDURANCE POINTS").arg(enduranceRestore);
         Items.append(i);
         return i;
     }
@@ -583,6 +628,16 @@ bool Player::RemoveItem(QString name)
     return false;
 }
 
+bool Player::RemoveItemByUUID(QString UUID)
+{
+    Item* i = GetItemByUUID(UUID);
+    if (i){
+        Items.removeOne(i);
+        return true;
+    }
+    return false;
+}
+
 QStringList Player::GetHiddenItems()
 {
     QStringList res;
@@ -641,6 +696,17 @@ Item* Player::GetItem(QString name)
     return NULL;
 }
 
+Item* Player::GetItemByUUID(QString UUID)
+{
+    if (!UUID.isEmpty()){
+        foreach(Item* i, Items){
+            if (QString::compare(i->GetUUID(), UUID) == 0)
+                return i;
+        }
+    }
+    return NULL;
+}
+
 int Player::SpecialItemsCount()
 {
     int res = 0;
@@ -665,28 +731,18 @@ void Player::SetHasBackpack(bool value)
 {
     m_Backpack = value;
     if (!m_Backpack){
-        QList<Item*> toRemove;
-        foreach (Item* i, Items){
+        QList<Item*>::iterator iterator;
+        for (iterator = Items.begin(); iterator != Items.end(); ++iterator){
+            Item* i = *iterator;
             if (i->GetIsBackpackItem())
-                toRemove.append(i);
-        }
-
-        foreach (Item* i, toRemove){
-            Items.removeOne(i);
-            delete i;
+                Items.erase(iterator);
         }
     }
 }
 
 bool Player::IsBackpackFull()
 {
-    int size = 0;
-    foreach(Item* i, Items){
-        if (i->GetIsBackpackItem())
-            size += i->GetSize();
-    }
-
-    return size >= 8;
+    return GetBackpackFreeSpace() <= 0;
 }
 
 int Player::GetBackpackFreeSpace()
@@ -727,7 +783,9 @@ QStringList Player::GetAllWeapons()
 
 Weapon* Player::GetEquippedWeapon()
 {
-    foreach (Item* i, Items){
+    QList<Item*>::const_iterator iterator;
+    for (iterator = Items.constBegin(); iterator != Items.constEnd(); ++iterator){
+        Item* i = *iterator;
         if (i->GetIsWeapon() && i->GetIsEquipped())
             return (Weapon*)i;
     }
@@ -737,7 +795,9 @@ Weapon* Player::GetEquippedWeapon()
 QList<Armor*> Player::GetEquippedArmors()
 {
     QList<Armor*> res;
-    foreach (Item* i, Items){
+    QList<Item*>::const_iterator iterator;
+    for (iterator = Items.constBegin(); iterator != Items.constEnd(); ++iterator){
+        Item* i = *iterator;
         if (i->GetIsArmor() && i->GetIsEquipped())
             res.append((Armor*)i);
     }
@@ -746,7 +806,9 @@ QList<Armor*> Player::GetEquippedArmors()
 
 Armor* Player::GetEquippedShield()
 {
-    foreach (Item* i, Items){
+    QList<Item*>::const_iterator iterator;
+    for (iterator = Items.constBegin(); iterator != Items.constEnd(); ++iterator){
+        Item* i = *iterator;
         if (i->GetIsArmor() && i->GetArmorClass() == Armor::Shield && i->GetIsEquipped())
             return (Armor*)i;
     }
@@ -756,15 +818,20 @@ Armor* Player::GetEquippedShield()
 QStringList Player::GetAllItems()
 {
     QStringList res;
-    foreach (Item* i, Items)
+    QList<Item*>::const_iterator iterator;
+    for (iterator = Items.constBegin(); iterator != Items.constEnd(); ++iterator){
+        Item* i = *iterator;
         res.append(i->Name);
+    }
     return res;
 }
 
 QStringList Player::GetSpecialItems()
 {
     QStringList res;
-    foreach (Item* i, Items){
+    QList<Item*>::const_iterator iterator;
+    for (iterator = Items.constBegin(); iterator != Items.constEnd(); ++iterator){
+        Item* i = *iterator;
         if (i->GetIsSpecial())
             res.append(i->Name);
     }
@@ -774,7 +841,9 @@ QStringList Player::GetSpecialItems()
 QStringList Player::GetItems()
 {
     QStringList res;
-    foreach (Item* i, Items){
+    QList<Item*>::const_iterator iterator;
+    for (iterator = Items.constBegin(); iterator != Items.constEnd(); ++iterator){
+        Item* i = *iterator;
         if (i->GetIsBackpackItem())
             res.append(i->Name);
     }
@@ -784,7 +853,9 @@ QStringList Player::GetItems()
 QStringList Player::GetMeals()
 {
     QStringList res;
-    foreach (Item* i, Items){
+    QList<Item*>::const_iterator iterator;
+    for (iterator = Items.constBegin(); iterator != Items.constEnd(); ++iterator){
+        Item* i = *iterator;
         if (i->GetIsMeal())
             res.append(i->Name);
     }
@@ -976,7 +1047,9 @@ void Player::ClearProperties()
 
 bool Player::HasQuiver()
 {
-    foreach (Item* i, Items){
+    QList<Item*>::const_iterator iterator;
+    for (iterator = Items.constBegin(); iterator != Items.constEnd(); ++iterator){
+        Item* i = *iterator;
         if (i->GetIsSpecial() && i->GetIsQuiver())
             return true;
     }
@@ -1071,6 +1144,64 @@ bool Player::CanEquipArmor(Armor* armor, QString* error)
     return true;
 }
 
+bool Player::CanTakeItem(Item* item, QString* error,
+                         int deltaWeapons, int deltaItems, int deltaSpecial, int deltaArrows)
+{
+    if (item->GetIsWeapon() && !item->GetIsSpecial()){
+        //Take weapon:
+        if (Weapons().count() + deltaWeapons + 1 > 2){
+            if (error)
+                *error = "You cannot carry more than two weapons.";
+            return false;
+        }else
+            return true;
+    }
+
+    if (item->GetIsSpecial()){
+        //Take special item:
+        if (SpecialItemsCount() + item->GetQuantity() + deltaSpecial > 12){
+            if (error)
+                *error = "You cannot carry more special items.";
+            return false;
+        }else
+            return true;
+    }else{
+        //Take arrows:
+        if (item->GetIsArrow()){
+            if (!HasQuiver()){
+                if (error)
+                    *error = "You don't have a quiver and cannot carry arrows.";
+                return false;
+            }
+            if (GetArrows() + deltaArrows + item->GetQuantity() > GetMaxArrows()){
+                if (error)
+                    *error = "You cannot carry more arrows.";
+                return false;
+            }
+            return true;
+        }else{
+            //Take normal item:
+            if (!HasBackpack() && item->GetIsBackpack()){
+                return true;
+            }else if (!HasBackpack()){
+                if (error)
+                    *error = "You don't have the backpack and cannot carry any item.";
+                return false;
+            }else if (HasBackpack() && item->GetIsBackpack()){
+                if (error)
+                    *error = "You have the backpack and cannot carry another one.";
+                return false;
+            }else if (GetBackpackFreeSpace() - deltaItems - item->GetTotalSize() < 0){
+                if (error)
+                    *error = "You cannot carry more backpack items.";
+                return false;
+            }else
+                return true;
+        }
+    }
+    return false;
+}
+
 void Player::Dump()
 {
     qDebug() << "Base properties:";
