@@ -1,8 +1,5 @@
 #include "fightwindow.h"
 #include "ui_fightwindow.h"
-#ifdef Q_WS_MAEMO_5
-#include <QMaemo5Style>
-#endif
 #include <QMessageBox>
 #include "commonstatic.h"
 #include "combatskillwindow.h"
@@ -22,10 +19,7 @@ FightWindow::FightWindow(QWidget *parent, Player* player, Enemy* enemy, bool can
     setAttribute(Qt::WA_Maemo5AutoOrientation, true);
     setAttribute(Qt::WA_Maemo5StackedWindow);
 
-    QColor secondaryColor = QMaemo5Style::standardColor("SecondaryTextColor");
-    QPalette secondaryPalette(QColor(255,255,255, 0));
-    secondaryPalette.setColor(QPalette::Foreground, secondaryColor);
-
+    QPalette secondaryPalette = CommonStatic::GetSecondaryPalette();
     ui->lblLoneWolf->setPalette(secondaryPalette);
     ui->lblEnemy->setPalette(secondaryPalette);
 #endif
@@ -34,6 +28,7 @@ FightWindow::FightWindow(QWidget *parent, Player* player, Enemy* enemy, bool can
     connect(ui->lblCombatSkillD, SIGNAL(linkActivated(QString)), this, SLOT(CombatSkillLinkSlot(QString)));
 
     SetCanClose(false);
+    m_BookWindow = NULL;
     m_UpdatingData = false;
     m_PlayerPointsLost = 0;
     m_EnemyPointsLost = 0;
@@ -301,7 +296,9 @@ void FightWindow::Evaluate(int number)
 
 bool FightWindow::ExecuteScript(QString name, QString script)
 {
-    QScriptValue objectValue = m_Engine->newQObject(this);
+    QScriptValue objectValue = m_Engine->newQObject(m_BookWindow);
+    m_Engine->globalObject().setProperty("MainWindow", objectValue);
+    objectValue = m_Engine->newQObject(this);
     m_Engine->globalObject().setProperty("Window", objectValue);
     objectValue = m_Engine->newQObject(m_Enemy);
     m_Engine->globalObject().setProperty("Enemy", objectValue);
