@@ -2,6 +2,7 @@
 #define FIGHTWINDOW_H
 
 #include <QMainWindow>
+#include "bookwindow.h"
 #include "randomnumberpicker.h"
 #include "controls/dialogwindow.h"
 #include "core/player.h"
@@ -24,9 +25,12 @@ public:
     explicit FightWindow(QWidget *parent = 0, Player* player=0, Enemy* enemy=0, bool canEvade=false, QScriptEngine* engine=0);
     ~FightWindow();
 
+    void SetBookWindow(BookWindow* value) { m_BookWindow = value; }
+
 private:
     Ui::FightWindow *ui;
 
+    BookWindow* m_BookWindow;
     bool m_UpdatingData;
     int m_LastRandomNumber;
     RandomNumberPicker* m_RandomPicker;
