@@ -2,7 +2,7 @@
 #include "ui_pickitemswindow.h"
 #include "commonstatic.h"
 
-PickItemsWindow::PickItemsWindow(QWidget *parent, Player* player, int serie, int goldCrowns, QList<Item*>* items, int max) :
+PickItemsWindow::PickItemsWindow(QWidget *parent, Section* section, Player* player, int serie, int goldCrowns, QList<Item*>* items, int max) :
     DialogWindow(parent),
     ui(new Ui::PickItemsWindow)
 {
@@ -10,13 +10,20 @@ PickItemsWindow::PickItemsWindow(QWidget *parent, Player* player, int serie, int
 #ifdef Q_WS_MAEMO_5
     setAttribute(Qt::WA_Maemo5AutoOrientation);
     setAttribute(Qt::WA_Maemo5StackedWindow);
+
+    QPalette secondaryPalette = CommonStatic::GetSecondaryPalette();
+    ui->lblPossession->setPalette(secondaryPalette);
+    ui->lblWeapons->setPalette(secondaryPalette);
+    ui->lblItems->setPalette(secondaryPalette);
+    ui->lblSpecialItems->setPalette(secondaryPalette);
 #endif
 
+    m_Section = section;
     m_Serie = serie;
     m_Player = player;
     m_GoldCrowns = goldCrowns;
     m_SelectedGoldCrowns = 0;
-    m_List = items;
+    m_ItemsList = items;
     m_Max = max;
 
     if (max == 0)
@@ -31,9 +38,6 @@ PickItemsWindow::PickItemsWindow(QWidget *parent, Player* player, int serie, int
     connect(ui->pbnDropWeapon1, SIGNAL(clicked()), this, SLOT(DropWeapon1Slot()));
     connect(ui->pbnDropWeapon2, SIGNAL(clicked()), this, SLOT(DropWeapon2Slot()));
 
-    //if (max > 0)
-    //    ui->pbnDone->setEnabled(false);
-
     m_RandomPicker = new RandomNumberPicker(this, ui->lblDice);
     connect(m_RandomPicker, SIGNAL(Random(int)), SLOT(RandomSlot(int)));
 
@@ -48,7 +52,6 @@ PickItemsWindow::~PickItemsWindow()
 void PickItemsWindow::ShowData(bool full)
 {
     int row = 0;
-    QLayoutItem* child;
     QSizePolicy stretchZero;
     stretchZero.setHorizontalStretch(0);
 
@@ -57,10 +60,7 @@ void PickItemsWindow::ShowData(bool full)
     ui->pbnDropWeapon2->setMaximumSize(120, 50);
 #endif
     if (full){
-        while ((child = ui->layPickItems->takeAt(0)) != 0) {
-            delete child->widget();
-            delete child;
-        }
+        CommonStatic::ClearLayout(ui->layPickItems);
 
         if (m_GoldCrowns){
             m_GoldCrownsLabel = new QLabel(this);
@@ -92,7 +92,7 @@ void PickItemsWindow::ShowData(bool full)
         }
 
         int index = 0;
-        foreach(Item* i, *m_List){
+        foreach(Item* i, *m_ItemsList){
             QLabel* lbl = new QLabel(this);
             if (i->GetQuantity() > 1)
                 lbl->setText(QString("%1 (%2)").arg(i->Name).arg(i->GetQuantity()));
@@ -148,64 +148,12 @@ void PickItemsWindow::ShowData(bool full)
         ui->lblWeapon2->setText("-");
     }
 
-    while ((child = ui->layItems->takeAt(0)) != 0) {
-        delete child->widget();
-        delete child;
-    }
-
-    row = 0;
-    int index = 0;
-    if (!m_Player->HasBackpack()){
-        QLabel* lbl = new QLabel(this);
-        lbl->setText("You don't have the backpack!");
-        ui->layItems->addWidget(lbl, 0, 0);
-    }else{
-        foreach(Item* i, m_Player->Items){
-            if (i->GetIsBackpackItem()){
-                QLabel* lbl = new QLabel(this);
-                lbl->setSizePolicy(stretchZero);
-                lbl->setText(QString("<b>%1:</b>").arg(row + 1));
-                ui->layItems->addWidget(lbl, row, 0);
-
-                lbl = new QLabel(this);
-                lbl->setText(i->Name);
-                lbl->setWordWrap(true);
-                ui->layItems->addWidget(lbl, row, 1);
-
-                lbl = new QLabel(this);
-                lbl->setText(i->Description);
-                lbl->setWordWrap(true);
-                ui->layItems->addWidget(lbl, row, 2);
-
-                QPushButton* pbn = new QPushButton(this);
-                pbn->setObjectName(QString::number(index));
-                pbn->setText("Discard");
-#ifdef Q_WS_MAEMO_5
-                pbn->setMaximumSize(120, 50);
-#else
-                pbn->setMaximumSize(75, 50);
-#endif
-                connect(pbn, SIGNAL(clicked()), this, SLOT(DropItemSlot()));
-                ui->layItems->addWidget(pbn, row, 3);
-
-                row++;
-                for (int j=1; j<i->GetSize(); j++){
-                    QLabel* lbl = new QLabel(this);
-                    lbl->setSizePolicy(stretchZero);
-                    lbl->setText(QString("<b>%1:</b>").arg(row + 1));
-                    ui->layItems->addWidget(lbl, row, 0);
-
-                    lbl = new QLabel(this);
-                    lbl->setText("-");
-                    lbl->setWordWrap(true);
-                    ui->layItems->addWidget(lbl, row, 1);
-
-                    row++;
-                }
-            }
-            index++;
-        }
-    }
+    CommonStatic::PopulateBackpackItemsLayout(this, m_Player, ui->layItems,
+                                              SLOT(DropItemSlot()), "Discard",
+                                              "", "");
+    CommonStatic::PopulateSpecialItemsLayout(this, m_Player, ui->laySpecialItems,
+                                             SLOT(DropItemSlot()), "Discard",
+                                             "");
 }
 
 void PickItemsWindow::CheckItemSlot(bool value)
@@ -217,49 +165,21 @@ void PickItemsWindow::CheckItemSlot(bool value)
             m_SelectedGoldCrowns = 0;
     }else{
         int idx = ((QWidget*)sender())->objectName().toInt();
-        Item* i = m_List->at(idx);
+        Item* i = m_ItemsList->at(idx);
 
         QCheckBox* chk = (QCheckBox*)sender();
         if (value){
-            Weapon* w = qobject_cast<Weapon*>(i);
-            if (w){
-                if (m_Player->Weapons().count() + CountSelectedWeapons() >= 2){
-                    QMessageBox msgBox(this);
-                    msgBox.setText("You cannot carry more than two weapons.");
-                    msgBox.exec();
-                    chk->setChecked(false);
-                    return;
-                }
-            }else{
-                if (i->GetIsBackpack() && m_Player->HasBackpack()){
-                    QMessageBox msgBox(this);
-                    msgBox.setText("You have the backpack and cannot carry another one.");
-                    msgBox.exec();
-                    chk->setChecked(false);
-                    return;
-                }else if (!i->GetIsSpecial()){
-                    if (i->GetIsArrow()){
-                        if (!m_Player->HasQuiver()){
-                            QMessageBox msgBox(this);
-                            msgBox.setText("You don't have a quiver and cannot carry arrows.");
-                            msgBox.exec();
-                            chk->setChecked(false);
-                            return;
-                        }
-                    }else if (!HasBackpack()){
-                        QMessageBox msgBox(this);
-                        msgBox.setText("You don't have the backpack and cannot carry any item.");
-                        msgBox.exec();
-                        chk->setChecked(false);
-                        return;
-                    }else if (m_Player->GetBackpackFreeSpace() - SelectedItemsSize() - i->GetTotalSize() < 0){
-                        QMessageBox msgBox(this);
-                        msgBox.setText("You cannot carry more items.");
-                        msgBox.exec();
-                        chk->setChecked(false);
-                        return;
-                    }
-                }
+            QString error;
+            if (!m_Player->CanTakeItem(i, &error,
+                                       CountSelectedWeapons(),
+                                       SelectedItemsSize(),
+                                       CountSelectedSpecialItems(),
+                                       CountSelectedArrows())){
+                QMessageBox msgBox(this);
+                msgBox.setText(error);
+                msgBox.exec();
+                chk->setChecked(false);
+                return;
             }
             m_Selected.append(i);
         }else
@@ -272,11 +192,9 @@ void PickItemsWindow::CheckItemSlot(bool value)
 
     if (m_Max > 0){
         if (count == m_Max){
-            //ui->pbnDone->setEnabled(true);
             foreach(QCheckBox* c, m_Checkboxes)
                 c->setEnabled(c->isChecked());
         }else{
-            //ui->pbnDone->setEnabled(false);
             foreach(QCheckBox* c, m_Checkboxes)
                 c->setEnabled(true);
         }
@@ -329,7 +247,7 @@ bool PickItemsWindow::DropWeapon(Weapon* weapon)
         weapon->Unequip();
         m_Player->CheckEndurance();
         m_Player->Items.removeOne(weapon);
-        delete weapon;
+        m_Section->Items.append(weapon);
         return true;
     }
     return false;
@@ -339,8 +257,7 @@ int PickItemsWindow::CountSelectedWeapons()
 {
     int res = 0;
     foreach (Item*i, m_Selected){
-        Weapon* w = qobject_cast<Weapon*>(i);
-        if (w)
+        if (i->GetIsWeapon() && !i->GetIsSpecial())
             res++;
     }
 
@@ -351,13 +268,35 @@ int PickItemsWindow::SelectedItemsSize()
 {
     int res = 0;
     foreach (Item*i, m_Selected){
-        if (!i->GetIsSpecial())
+        if (i->GetIsBackpackItem())
             res += i->GetTotalSize();
     }
 
     return res;
 }
 
+int PickItemsWindow::CountSelectedSpecialItems()
+{
+    int res = 0;
+    foreach (Item*i, m_Selected){
+        if (i->GetIsSpecial())
+            res++;
+    }
+
+    return res;
+}
+
+int PickItemsWindow::CountSelectedArrows()
+{
+    int res = 0;
+    foreach (Item*i, m_Selected){
+        if (i->GetIsArrow())
+            res += i->GetQuantity();
+    }
+
+    return res;
+}
+
 void PickItemsWindow::DropItemSlot()
 {
     int idx = ((QWidget*)sender())->objectName().toInt();
@@ -373,7 +312,7 @@ bool PickItemsWindow::DropItem(Item* item)
         item->Unequip();
         m_Player->CheckEndurance();
         m_Player->Items.removeOne(item);
-        delete item;
+        m_Section->Items.append(item);
         return true;
     }
     return false;
