@@ -16,7 +16,7 @@ class PickItemsWindow : public DialogWindow
     Q_OBJECT
 
 public:
-    explicit PickItemsWindow(QWidget *parent = 0, Player* player=0, int serie=Book::Kai, int goldCrowns=0, QList<Item*>* items=0, int max=2);
+    explicit PickItemsWindow(QWidget *parent = 0, Section* section=0, Player* player=0, int serie=Book::Kai, int goldCrowns=0, QList<Item*>* items=0, int max=2);
     ~PickItemsWindow();
 
     int GetPickedItemsCount() { return m_Selected.count(); }
@@ -28,9 +28,10 @@ private:
     QPushButton* m_GoldCrownsButton;
     RandomNumberPicker* m_RandomPicker;
     int m_Serie;
+    Section* m_Section;
     Player* m_Player;
     int m_GoldCrowns;
-    QList<Item*>* m_List;
+    QList<Item*>* m_ItemsList;
     int m_Max;
     int m_SelectedGoldCrowns;
     QList<Item*> m_Selected;
@@ -41,6 +42,8 @@ private:
     bool DropItem(Item* item);
     int CountSelectedWeapons();
     int SelectedItemsSize();
+    int CountSelectedSpecialItems();
+    int CountSelectedArrows();
     bool HasBackpack();
 
 private slots:
