@@ -17,6 +17,12 @@ ShopWindow::ShopWindow(QWidget *parent, Player* player, Section* section) :
 #ifdef Q_WS_MAEMO_5
     setAttribute(Qt::WA_Maemo5AutoOrientation);
     setAttribute(Qt::WA_Maemo5StackedWindow);
+
+    QPalette secondaryPalette = CommonStatic::GetSecondaryPalette();
+    ui->lblPossession->setPalette(secondaryPalette);
+    ui->lblWeapons->setPalette(secondaryPalette);
+    ui->lblItems->setPalette(secondaryPalette);
+    ui->lblSpecialItems->setPalette(secondaryPalette);
 #endif
 
     m_Player = player;
@@ -36,14 +42,10 @@ ShopWindow::~ShopWindow()
 void ShopWindow::ShowData()
 {
     int row = 0;
-    QLayoutItem* child;
     QSizePolicy stretchZero;
     stretchZero.setHorizontalStretch(0);
 
-    while ((child = ui->layPickItems->takeAt(0)) != 0) {
-        delete child->widget();
-        delete child;
-    }
+    CommonStatic::ClearLayout(ui->layPickItems);
 
     QLabel* lbl = new QLabel(this);
     lbl->setText("<b>Item</b>");
@@ -142,53 +144,28 @@ void ShopWindow::ShowData()
         ui->lblWeapon2->setText("-");
     }
 
-    while ((child = ui->layItems->takeAt(0)) != 0) {
-        delete child->widget();
-        delete child;
+    CommonStatic::ClearLayout(ui->layItems);
+
+    // Set the sell price (used by PopulateBackpackItemsLayout)
+    QList<Item*>::const_iterator iterator;
+    for (iterator = m_Player->Items.constBegin(); iterator != m_Player->Items.constEnd(); ++iterator){
+        Item* i = *iterator;
+        if (i->GetIsBackpackItem())
+            i->SetSellPrice(GetSellPrice(i));
     }
 
-    row = 0;
-    index = 0;
-    if (!m_Player->HasBackpack()){
-        QLabel* lbl = new QLabel(this);
-        lbl->setText("You don't have the backpack!");
-        ui->layItems->addWidget(lbl, 0, 0);
-    }else{
-        foreach(Item* i, m_Player->Items){
-            if (i->GetIsBackpackItem()){
-                QLabel* lbl = new QLabel(this);
-                lbl->setSizePolicy(stretchZero);
-                lbl->setText(QString("<b>%1:</b>").arg(row + 1));
-                ui->layItems->addWidget(lbl, row, 0);
+    CommonStatic::PopulateBackpackItemsLayout(this, m_Player, ui->layItems,
+                                              SLOT(DropItemSlot()), "Discard",
+                                              "", SLOT(SellItemSlot()));
+    CommonStatic::PopulateSpecialItemsLayout(this, m_Player, ui->laySpecialItems,
+                                             SLOT(DropItemSlot()), "Discard",
+                                             "");
 
-                lbl = new QLabel(this);
-                lbl->setText(i->Name);
-                lbl->setWordWrap(true);
-                ui->layItems->addWidget(lbl, row, 1);
-
-                lbl = new QLabel(this);
-                lbl->setText(i->Description);
-                lbl->setWordWrap(true);
-                ui->layItems->addWidget(lbl, row, 2);
-
-                QPushButton* pbn = new QPushButton(this);
-                pbn->setObjectName(QString::number(index));
-
-                int sell = GetSellPrice(i);
-                if (sell){
-                    pbn->setText("Sell");
-                    connect(pbn, SIGNAL(clicked()), this, SLOT(SellItemSlot()));
-                }else{
-                    pbn->setText("Discard");
-                    connect(pbn, SIGNAL(clicked()), this, SLOT(DropItemSlot()));
-                }
-                pbn->setMaximumSize(75, 50);
-                ui->layItems->addWidget(pbn, row, 3);
-
-                row++;
-            }
-            index++;
-        }
+    // Reset the sell price:
+    for (iterator = m_Player->Items.constBegin(); iterator != m_Player->Items.constEnd(); ++iterator){
+        Item* i = *iterator;
+        if (i->GetIsBackpackItem())
+            i->SetSellPrice(0);
     }
 }
 
@@ -196,54 +173,26 @@ void ShopWindow::BuyItemSlot()
 {
     int idx = ((QWidget*)sender())->objectName().toInt();
     ShopItem* i = m_Section->ShopItems.at(idx);
-
-    if (i->IsWeapon() && m_Player->WeaponsCount() == 2){
+    QString error;
+    if (!m_Player->CanTakeItem(i, &error)){
         QMessageBox msgBox(this);
-        msgBox.setText("You cannot carry more than two weapons.");
+        msgBox.setText(error);
         msgBox.exec();
         return;
-    }else{
-        if (i->GetIsArrow() && !m_Player->HasQuiver()){
-            QMessageBox msgBox(this);
-            msgBox.setText("You don't have the quiver and cannot carry arrows.");
-            msgBox.exec();
-            return;
-        }
-
-        if (i->GetIsArrow() && m_Player->GetArrows() == m_Player->GetMaxArrows()){
-            QMessageBox msgBox(this);
-            msgBox.setText("You cannot carry more arrows.");
-            msgBox.exec();
-            return;
-        }
-
-        if (!i->GetIsSpecial() && !m_Player->HasBackpack()){
-            QMessageBox msgBox(this);
-            msgBox.setText("You don't have the backpack and cannot carry any item.");
-            msgBox.exec();
-            return;
-        }
-
-        if (!i->GetIsSpecial() && m_Player->IsBackpackFull()){
-            QMessageBox msgBox(this);
-            msgBox.setText("You cannot carry more items.");
-            msgBox.exec();
-            return;
-        }
     }
 
     if (CommonStatic::ConfirmMessage(this, "Confirm", QString("Buy a %1 for %2 Gold Crown(s)?").arg(i->Name).arg(i->Price)) == QMessageBox::Yes){
         m_Player->SetGoldCrowns(m_Player->GetGoldCrowns() - i->Price);
         qDebug() << "IsWeapon=" << i->IsWeapon();
         if (i->IsWeapon())
-            m_Player->Items.append((Weapon*)i);
+            m_Player->Items.append((Weapon*)new Item(*i));
         else{
             if (i->GetIsArrow())
                 m_Player->SetArrows(m_Player->GetArrows() + i->GetQuantity());
             else if (i->GetIsBackpack())
                 m_Player->SetHasBackpack(true);
             else
-                m_Player->Items.append((Item*)i);
+                m_Player->Items.append(new Item(*i));
         }
         if (!i->UnlimitedQuantity)
             m_Section->ShopItems.removeOne(i);
