@@ -2,9 +2,6 @@
 #include "ui_statuswindow.h"
 
 #include <QPushButton>
-#ifdef Q_WS_MAEMO_5
-#include <QMaemo5Style>
-#endif
 #include "commonstatic.h"
 #include "ruleswindow.h"
 #include "combatskillwindow.h"
@@ -27,10 +24,7 @@ StatusWindow::StatusWindow(QWidget *parent, Player* player, Section* section, QS
     setAttribute(Qt::WA_Maemo5AutoOrientation, true);
     setAttribute(Qt::WA_Maemo5StackedWindow);
 
-    QColor secondaryColor = QMaemo5Style::standardColor("SecondaryTextColor");
-    QPalette secondaryPalette(QColor(255,255,255, 0));
-    secondaryPalette.setColor(QPalette::Foreground, secondaryColor);
-
+    QPalette secondaryPalette = CommonStatic::GetSecondaryPalette();
     ui->lblSectionItems->setPalette(secondaryPalette);
     ui->lblStatus->setPalette(secondaryPalette);
     ui->lblDisciplines->setPalette(secondaryPalette);
@@ -70,6 +64,7 @@ StatusWindow::~StatusWindow()
 
 void StatusWindow::ShowData()
 {
+    QList<Item*>::const_iterator iterator;
     ui->lblNotes->setText(m_Player->GetNotes());
 
     switch (m_Serie){
@@ -101,23 +96,25 @@ void StatusWindow::ShowData()
         ui->lblSectionItems->setVisible(false);
         ui->lineSection->setVisible(false);
 
-        ClearLayout(ui->laySectionCrowns);
-        ClearLayout(ui->laySectionItems);
+        CommonStatic::ClearLayout(ui->laySectionCrowns);
+        CommonStatic::ClearLayout(ui->laySectionItems);
     }else{
         ui->lblSectionItems->setVisible(true);
 
-        ClearLayout(ui->laySectionItems);
+        CommonStatic::ClearLayout(ui->laySectionItems);
         if (m_Section->GoldCrowns > 0){
             ui->lblSecGoldCrowns->setText(QString::number(m_Section->GoldCrowns));
             if (m_Player->GoldCrowns == 50)
                 ui->pbnPickGoldCrowns->setEnabled(false);
         }else{
-            ClearLayout(ui->laySectionCrowns);
+            CommonStatic::ClearLayout(ui->laySectionCrowns);
         }
 
         int row = 1;
-        foreach(Item* i, m_Section->Items){
+        for (iterator = m_Section->Items.constBegin(); iterator != m_Section->Items.constEnd(); ++iterator){
+            Item* i = *iterator;
             QLabel* lbl = new QLabel(this);
+            lbl->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
             lbl->setText(i->Name);
             lbl->setWordWrap(true);
             ui->laySectionItems->addWidget(lbl, row, 0);
@@ -190,16 +187,19 @@ void StatusWindow::ShowData()
         ui->lblWeapon2->setText("-");
     }
 
-    ClearLayout(ui->layEquippedWeapons);
+    CommonStatic::ClearLayout(ui->layEquippedWeapons);
     int row = 0;
-    foreach(Item* w, m_Player->Items){
+    for (iterator = m_Player->Items.constBegin(); iterator != m_Player->Items.constEnd(); ++iterator){
+        Item* w = *iterator;
         if (w->GetIsWeapon() && w->GetIsEquipped()){
             QLabel* lbl = new QLabel(this);
+            lbl->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
             lbl->setText(w->Name);
             lbl->setWordWrap(true);
             ui->layEquippedWeapons->addWidget(lbl, row, 0);
 
             lbl = new QLabel(this);
+            lbl->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
             lbl->setText(w->GetDescription());
             lbl->setWordWrap(true);
             ui->layEquippedWeapons->addWidget(lbl, row, 1);
@@ -213,16 +213,19 @@ void StatusWindow::ShowData()
         ui->layEquippedWeapons->addWidget(lbl, row, 0);
     }
 
-    ClearLayout(ui->layEquippedArmor);
+    CommonStatic::ClearLayout(ui->layEquippedArmor);
     row = 0;
-    foreach(Item* i, m_Player->Items){
+    for (iterator = m_Player->Items.constBegin(); iterator != m_Player->Items.constEnd(); ++iterator){
+        Item* i = *iterator;
         if (i->GetIsArmor() && i->GetIsEquipped()){
             QLabel* lbl = new QLabel(this);
+            lbl->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
             lbl->setText(i->Name);
             lbl->setWordWrap(true);
             ui->layEquippedArmor->addWidget(lbl, row, 0);
 
             lbl = new QLabel(this);
+            lbl->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
             lbl->setText(i->GetDescription());
             lbl->setWordWrap(true);
             ui->layEquippedArmor->addWidget(lbl, row, 1);
@@ -236,14 +239,16 @@ void StatusWindow::ShowData()
         ui->layEquippedArmor->addWidget(lbl, row, 0);
     }
 
-    ClearLayout(ui->layKaiDisciplines);
-    ClearLayout(ui->layWeaponChecklist);
+    CommonStatic::ClearLayout(ui->layKaiDisciplines);
+    CommonStatic::ClearLayout(ui->layWeaponChecklist);
     QSizePolicy stretchZero;
     stretchZero.setHorizontalStretch(0);
     QSizePolicy stretchOne;
     stretchOne.setHorizontalStretch(1);
     row = 0;
-    foreach (KaiDiscipline* d, m_Player->KaiDisciplines){
+    QList<KaiDiscipline*>::const_iterator dIterator;
+    for (dIterator = m_Player->KaiDisciplines.constBegin(); dIterator != m_Player->KaiDisciplines.constEnd(); ++dIterator){
+        KaiDiscipline* d = *dIterator;
         if (d->Serie != m_Serie)
             continue;
 
@@ -288,142 +293,13 @@ void StatusWindow::ShowData()
         }
     }
 
-    ClearLayout(ui->layItems);
-    row = 0;
-    int index = 0;
+    CommonStatic::PopulateBackpackItemsLayout(this, m_Player, ui->layItems,
+                                              SLOT(DropItemSlot()), "Drop",
+                                              SLOT(UseItemSlot()), "");
+    CommonStatic::PopulateSpecialItemsLayout(this, m_Player, ui->laySpecialItems,
+                                             SLOT(DropItemSlot()), "Drop",
+                                             SLOT(EquipItemSlot()));
 
-    if (!m_Player->HasBackpack()){
-        QLabel* lbl = new QLabel(this);
-        lbl->setText("You don't have the backpack!");
-        ui->layItems->addWidget(lbl, 0, 0);
-    }else{
-        foreach(Item* i, m_Player->Items){
-            if (i->GetIsBackpackItem()){
-                QLabel* lbl = new QLabel(this);
-                lbl->setSizePolicy(stretchZero);
-                lbl->setText(QString("<b>%1:</b>").arg(row + 1));
-                ui->layItems->addWidget(lbl, row, 0);
-
-                lbl = new QLabel(this);
-                lbl->setText(i->Name);
-                lbl->setWordWrap(true);
-                ui->layItems->addWidget(lbl, row, 1);
-
-                lbl = new QLabel(this);
-                lbl->setText(i->GetDescription());
-                lbl->setWordWrap(true);
-                ui->layItems->addWidget(lbl, row, 2);
-
-                QPushButton* pbn = new QPushButton(this);
-                pbn->setObjectName(QString::number(index));
-                if (i->GetIsMeal())
-                    pbn->setText("Eat");
-                else
-                    pbn->setText("Use");
-                pbn->setEnabled(i->IsUsable());
-                pbn->setMaximumSize(75, 50);
-                connect(pbn, SIGNAL(clicked()), this, SLOT(UseItemSlot()));
-                ui->layItems->addWidget(pbn, row, 3);
-
-                pbn = new QPushButton(this);
-                pbn->setObjectName(QString::number(index));
-                pbn->setText("Drop");
-                pbn->setMaximumSize(75, 50);
-                connect(pbn, SIGNAL(clicked()), this, SLOT(DropItemSlot()));
-                ui->layItems->addWidget(pbn, row, 4);
-
-                row++;
-                for (int j=1; j<i->GetSize(); j++){
-                    QLabel* lbl = new QLabel(this);
-                    lbl->setSizePolicy(stretchZero);
-                    lbl->setText(QString("<b>%1:</b>").arg(row + 1));
-                    ui->layItems->addWidget(lbl, row, 0);
-
-                    lbl = new QLabel(this);
-                    lbl->setText("-");
-                    lbl->setWordWrap(true);
-                    ui->layItems->addWidget(lbl, row, 1);
-
-                    row++;
-                }
-            }
-            index++;
-        }
-
-        for (int i=row; i<8; i++){
-            QLabel* lbl = new QLabel(this);
-            lbl->setSizePolicy(stretchZero);
-            lbl->setText(QString("<b>%1:</b>").arg(i + 1));
-            ui->layItems->addWidget(lbl, i, 0);
-
-            lbl = new QLabel(this);
-            lbl->setText("");
-            lbl->setWordWrap(true);
-            ui->layItems->addWidget(lbl, row, 1);
-
-            QPushButton* pbn = new QPushButton(this);
-            pbn->setText("Use");
-            pbn->setEnabled(false);
-            pbn->setMaximumSize(75, 50);
-            ui->layItems->addWidget(pbn, i, 3);
-
-            pbn = new QPushButton(this);
-            pbn->setText("Drop");
-            pbn->setEnabled(false);
-            pbn->setMaximumSize(75, 50);
-            ui->layItems->addWidget(pbn, i, 4);
-
-        }
-    }
-
-    ClearLayout(ui->laySpecialItems);
-    row = 0;
-    index = 0;
-    foreach(Item* i, m_Player->Items){
-        if (i->GetIsSpecial()){
-            QLabel* lbl = new QLabel(this);
-            lbl->setText(i->Name);
-            lbl->setWordWrap(true);
-            ui->laySpecialItems->addWidget(lbl, row, 0);
-
-            lbl = new QLabel(this);
-            lbl->setText(i->GetDescription());
-            lbl->setWordWrap(true);
-            ui->laySpecialItems->addWidget(lbl, row, 1);
-
-            if(i->GetIsWeapon() || i->GetIsArmor()){
-                QPushButton* pbn = new QPushButton(this);
-                pbn->setEnabled(i->GetCanBeEquipped());
-                pbn->setObjectName(QString::number(index));
-                if (i->GetIsEquipped()){
-                    pbn->setText("Unequip");
-#ifdef Q_WS_MAEMO_5
-                    pbn->setMaximumSize(120, 50);
-#endif
-                }else{
-                    pbn->setText("Equip");
-#ifdef Q_WS_MAEMO_5
-                    pbn->setMaximumSize(75, 50);
-#endif
-                }
-#ifndef Q_WS_MAEMO_5
-                pbn->setMaximumSize(75, 50);
-#endif
-                connect(pbn, SIGNAL(clicked()), this, SLOT(EquipItemSlot()));
-                ui->laySpecialItems->addWidget(pbn, row, 2);
-            }
-
-            QPushButton* pbn = new QPushButton(this);
-            pbn->setObjectName(QString::number(index));
-            pbn->setText("Drop");
-            pbn->setMaximumSize(75, 50);
-            connect(pbn, SIGNAL(clicked()), this, SLOT(DropItemSlot()));
-            ui->laySpecialItems->addWidget(pbn, row, 3);
-
-            row++;
-        }
-        index++;
-    }
 }
 
 bool StatusWindow::EquipArmor(Armor* armor)
@@ -578,20 +454,21 @@ void StatusWindow::TakeItemSlot()
     int idx = ((QWidget*)sender())->objectName().toInt();
     Item* i = m_Section->Items.at(idx);
 
+    QString error;
+    if (!m_Player->CanTakeItem(i, &error)){
+        QMessageBox msgBox(this);
+        msgBox.setText(error);
+        msgBox.exec();
+        return;
+    }
+
     Weapon* w = qobject_cast<Weapon*>(i);
     if (w && !w->GetIsSpecial()){
         //Take weapon:
-        if (m_Player->Weapons().count() == 2){
-            QMessageBox msgBox(this);
-            msgBox.setText("You cannot carry more than two weapons.");
-            msgBox.exec();
-            return;
-        }else{
-            m_Player->Items.append(w);
-            m_Section->Items.removeOne(w);
-            ShowData();
-            return;
-        }
+        m_Player->Items.append(w);
+        m_Section->Items.removeOne(w);
+        ShowData();
+        return;
     }
 
     if (i->GetIsSpecial()){
@@ -603,19 +480,6 @@ void StatusWindow::TakeItemSlot()
     }else{
         //Take arrows:
         if (i->GetIsArrow()){
-            if (!m_Player->HasQuiver()){
-                QMessageBox msgBox(this);
-                msgBox.setText("You don't have a quiver and cannot carry arrows.");
-                msgBox.exec();
-                return;
-            }
-            if (m_Player->GetArrows() == m_Player->GetMaxArrows()){
-                QMessageBox msgBox(this);
-                msgBox.setText("You cannot carry more arrows.");
-                msgBox.exec();
-                return;
-            }
-
             m_Player->SetArrows(m_Player->GetArrows() + i->GetQuantity());
             m_Section->Items.removeOne(i);
             ShowData();
@@ -626,21 +490,6 @@ void StatusWindow::TakeItemSlot()
                 m_Section->Items.removeOne(i);
                 ShowData();
                 return;
-            }else if (!m_Player->HasBackpack()){
-                QMessageBox msgBox(this);
-                msgBox.setText("You don't have the backpack and cannot carry any item.");
-                msgBox.exec();
-                return;
-            }else if (m_Player->HasBackpack() && i->GetIsBackpack()){
-                QMessageBox msgBox(this);
-                msgBox.setText("You have the backpack and cannot carry another one.");
-                msgBox.exec();
-                return;
-            }else if (m_Player->IsBackpackFull()){
-                QMessageBox msgBox(this);
-                msgBox.setText("You cannot carry more items.");
-                msgBox.exec();
-                return;
             }else{
                 m_Player->Items.append(i);
                 m_Section->Items.removeOne(i);
@@ -677,20 +526,12 @@ void StatusWindow::UseItemSlot()
     if (res == QMessageBox::Yes){
         i->Use(m_Engine, m_Functions);
         m_UsedItems.append(i);
-        m_Player->Items.removeOne(i);
+        if (i->GetUsability() <= 0)
+            m_Player->Items.removeOne(i);
         ShowData();
     }
 }
 
-void StatusWindow::ClearLayout(QLayout* layout)
-{
-    QLayoutItem* child;
-    while ((child = layout->takeAt(0)) != 0) {
-        delete child->widget();
-        delete child;
-    }
-}
-
 void StatusWindow::DiscipleLinkSlot(QString link)
 {
     RulesWindow* dlg = new RulesWindow(this, m_Book->BaseDirectory);
